use chrono::Local;
use dataworker::diesel::{Connection, PgConnection};
use dataworker::dtos::Root;
use dataworker::{insert_document, WilldatenError};
use dotenvy::dotenv;
use std::env;
use std::io::Write;

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    dotenv().ok();
    let stations: Vec<_> = env::var("STATIONS")?
        .split(',')
        .map(|s| s.to_string())
        .collect();
    let token = env::var("VRN_TOKEN")?;
    let req = env::var("REQ")?;
    let db_url = env::var("DATABASE_URL")?;
    let mut conn = PgConnection::establish(&db_url)?;

    let total = stations.len();
    let mut count = 0;

    let client = reqwest::Client::new();

    for station in stations {
        count += 1;
        let date = Local::now();
        let req = req
            .replace("TEMPLATE_ID", &station)
            .replace("TEMPLATE_DATE", &date.format("%Y%m%d").to_string())
            .replace("TEMPLATE_DATE", &date.format("%H%M").to_string());
        print!("\x1B[2K\rFetching {}/{} ({}) ", count, total, station);
        std::io::stdout().flush().unwrap();
        if let Ok(doc) = client
            .get(req)
            .header("Authorization", &token)
            .send()
            .await?
            .json::<Root>()
            .await {
            let place_name = doc.dm.itd_odv_assigned_stops.name_with_place.clone();
            match insert_document(doc,
                &mut conn,
            ) {
                Ok(_) => {}
                Err(ref e) if e.downcast_ref::<WilldatenError>().is_some() => println!("\x1B[2K\rMissing arrivals for {} ({})!", station, place_name),
                Err(e) => println!("\x1B[2K\rError inserting document: {}", e),
            }
        } else { println!("\x1B[2K\rFailed to fetch station {}", station); }
    }
    println!();
    Ok(())
}
